@@ -20,6 +20,10 @@
  * MA 02111-1307, USA.
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "nettle-meta.h"
 
 #include "aes.h"
